@@ -4,6 +4,8 @@
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 
+;; This file is not part of GNU Emacs.
+
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 3, or (at your option)
@@ -47,8 +49,7 @@
 ;;     fields))
 
 (with-no-warnings (require 'cl))
-
-(declare-function haskell-read-directory-name "haskell-process.el" (prompt default))
+(require 'haskell-utils)
 
 (defconst haskell-cabal-general-fields
   ;; Extracted with (haskell-cabal-extract-fields-from-doc "general-fields")
@@ -99,12 +100,6 @@
 (defvar haskell-cabal-buffers nil
   "List of Cabal buffers.")
 
-;; (defsubst* inferior-haskell-string-prefix-p (str1 str2)
-;;   "Return non-nil if STR1 is a prefix of STR2"
-;;   (eq t (compare-strings str2 nil (length str1) str1 nil nil)))
-4
-(autoload 'derived-mode-p "derived")	; Emacs 21
-
 (defun haskell-cabal-buffers-clean (&optional buffer)
   (let ((bufs ()))
     (dolist (buf haskell-cabal-buffers)
@@ -130,7 +125,7 @@
   (set (make-local-variable 'comment-start) "-- ")
   (set (make-local-variable 'comment-start-skip) "\\(^[ \t]*\\)--[ \t]*")
   (set (make-local-variable 'comment-end) "")
-  (set (make-local-variable 'comment-end-skip) "[ 	]*\\(\\s>\\|\n\\)")
+  (set (make-local-variable 'comment-end-skip) "[ \t]*\\(\\s>\\|\n\\)")
   )
 
 (defun haskell-cabal-get-setting (name)
@@ -162,44 +157,87 @@
    and indeed just prompting the user. Do them all."
   (let* ((file (haskell-cabal-find-file))
          (dir (when file (file-name-directory file))))
-    (haskell-read-directory-name
+    (haskell-utils-read-directory-name
      (format "Cabal dir%s: " (if file (format " (guessed from %s)" (file-relative-name file)) ""))
      dir)))
 
-(defun haskell-cabal-compute-checksum (cabal-dir)
-  "Computes a checksum of the .cabal configuration files."
-  (let* ((cabal-file-paths (directory-files cabal-dir t "\\.cabal$"))
-         (get-file-contents (lambda (path)
-                              (with-temp-buffer (insert-file-contents path)
-                                                (buffer-string))))
-         (cabal-file-contents (map 'list get-file-contents cabal-file-paths))
-         (cabal-config (reduce 'concat cabal-file-contents)))
-    (md5 cabal-config)))
+(defun haskell-cabal-compute-checksum (dir)
+  "Compute MD5 checksum of package description file in DIR.
+Return nil if no Cabal description file could be located via
+`haskell-cabal-find-pkg-desc'."
+  (let ((cabal-file (haskell-cabal-find-pkg-desc dir)))
+    (when cabal-file
+      (with-temp-buffer
+        (insert-file-contents cabal-file)
+        (md5 (buffer-string))))))
 
-(defun haskell-cabal-find-file ()
-  "Return a buffer visiting the cabal file of the current directory, or nil."
+(defun haskell-cabal-find-file (&optional dir)
+  "Search for package description file upwards starting from DIR.
+If DIR is nil, `default-directory' is used as starting point for
+directory traversal.  Upward traversal is aborted if file owner
+changes.  Uses`haskell-cabal-find-pkg-desc' internally."
   (catch 'found
-    (let ((user (nth 2 (file-attributes default-directory)))
+    (let ((user (nth 2 (file-attributes (or dir default-directory))))
           ;; Abbreviate, so as to stop when we cross ~/.
-          (root (abbreviate-file-name default-directory))
-          files)
+          (root (abbreviate-file-name (or dir default-directory))))
+      ;; traverse current dir up to root as long as file owner doesn't change
       (while (and root (equal user (nth 2 (file-attributes root))))
-        (if (setq files (directory-files root 'full "\\.cabal\\'"))
-            ;; Avoid the .cabal directory.
-            (dolist (file files (throw 'found nil))
-              (unless (file-directory-p file)
-                (throw 'found file)))
-          (if (equal root
-                     (setq root (file-name-directory
-                                 (directory-file-name root))))
-              (setq root nil))))
+        (let ((cabal-file (haskell-cabal-find-pkg-desc root)))
+          (when cabal-file
+            (throw 'found cabal-file)))
+
+        (let ((proot (file-name-directory (directory-file-name root))))
+          (if (equal proot root) ;; fix-point reached?
+              (throw 'found nil)
+            (setq root proot))))
       nil)))
 
-(defun haskell-cabal-find-dir ()
-  "Use the .cabal file-finding function to find the Cabal dir."
-  (let ((file (haskell-cabal-find-file)))
-    (when file
-      (file-name-directory file))))
+(defun haskell-cabal-find-pkg-desc (dir &optional allow-multiple)
+  "Find a package description file in the directory DIR.
+Returns nil if none or multiple \".cabal\" files were found.  If
+ALLOW-MULTIPLE is non nil, in case of multiple \".cabal\" files,
+a list is returned instead of failing with a nil result."
+  ;; This is basically a port of Cabal's
+  ;; Distribution.Simple.Utils.findPackageDesc function
+  ;;  http://hackage.haskell.org/packages/archive/Cabal/1.16.0.3/doc/html/Distribution-Simple-Utils.html
+  ;; but without the exception throwing.
+  (let* ((cabal-files
+          (remove-if 'file-directory-p
+                     (remove-if-not 'file-exists-p
+                                    (directory-files dir t ".\\.cabal\\'")))))
+    (cond
+     ((= (length cabal-files) 1) (car cabal-files)) ;; exactly one candidate found
+     (allow-multiple cabal-files) ;; pass-thru multiple candidates
+     (t nil))))
+
+(defun haskell-cabal-find-dir (&optional dir)
+  "Like `haskell-cabal-find-file' but returns directory instead.
+See `haskell-cabal-find-file' for meaning of DIR argument."
+  (let ((cabal-file (haskell-cabal-find-file dir)))
+    (when cabal-file
+      (file-name-directory cabal-file))))
+
+;;;###autoload
+(defun haskell-cabal-visit-file (other-window)
+  "Locate and visit package description file for file visited by current buffer.
+This uses `haskell-cabal-find-file' to locate the closest
+\".cabal\" file and open it.  This command assumes a common Cabal
+project structure where the \".cabal\" file is in the top-folder
+of the project, and all files related to the project are in or
+below the top-folder.  If called with non-nil prefix argument
+OTHER-WINDOW use `find-file-other-window'."
+  (interactive "P")
+  ;; Note: We aren't allowed to rely on haskell-session here (which,
+  ;; in pathological cases, can have a different .cabal file
+  ;; associated with the current buffer)
+  (if buffer-file-name
+      (let ((cabal-file (haskell-cabal-find-file (file-name-directory buffer-file-name))))
+        (if cabal-file
+            (if other-window
+                (find-file-other-window cabal-file)
+              (find-file cabal-file))
+          (error "Could not locate \".cabal\" file for %S" buffer-file-name)))
+    (error "Cannot locate \".cabal\" file for buffers not visiting any file")))
 
 (defvar haskell-cabal-commands
   '("install"
