@@ -1,7 +1,7 @@
 ;;; haskell-doc.el --- show function types in echo area  -*- coding: utf-8 -*-
 
 ;; Copyright (C) 2004, 2005, 2006, 2007, 2009  Free Software Foundation, Inc.
-;; Copyright (C) 1997 Hans-Wolfgang Loidl
+;; Copyright (C) 1997  Hans-Wolfgang Loidl
 
 ;; Author: Hans-Wolfgang Loidl <hwloidl@dcs.glasgow.ac.uk>
 ;; Temporary Maintainer and Hacker: Graeme E Moss <gem@cs.york.ac.uk>
@@ -9,8 +9,7 @@
 ;; Created: 1997-06-17
 ;; URL: http://cvs.haskell.org/cgi-bin/cvsweb.cgi/fptools/CONTRIB/haskell-modes/emacs/haskell-doc.el?rev=HEAD
 
-;;; Copyright:
-;;  ==========
+;; This file is not part of GNU Emacs.
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -26,7 +25,6 @@
 ;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
-;;  ===========
 
 ;; This program shows the type of the Haskell function under the cursor in the
 ;; minibuffer.  It acts as a kind of "Emacs background process", by regularly
@@ -43,8 +41,7 @@
 
 ;; This program was inspired by the `eldoc.el' package by Noah Friedman.
 
-;;; Installation:
-;;  =============
+;; Installation:
 
 ;; One useful way to enable this minor mode is to put the following in your
 ;; .emacs:
@@ -56,8 +53,7 @@
 ;;     or
 ;;      (add-hook 'haskell-mode-hook 'turn-on-haskell-doc-mode) ; haskell-mode
 
-;;; Customisation:
-;;  ==============
+;; Customisation:
 
 ;; You can control what exactly is shown by setting the following variables to
 ;; either t or nil:
@@ -73,10 +69,10 @@
 ;;
 ;;   (setq haskell-doc-show-user-defined t)
 ;;   (setq haskell-doc-user-defined-ids
-;;	(list
-;;	   '("main" . "just another pathetic main function")
-;;	   '("foo" . "a very dummy name")
-;;	   '("bar" . "another dummy name")))
+;;      (list
+;;         '("main" . "just another pathetic main function")
+;;         '("foo" . "a very dummy name")
+;;         '("bar" . "another dummy name")))
 
 ;;  The following two variables are useful to make the type fit on one line:
 ;;  If `haskell-doc-chop-off-context' is non-nil the context part of the type
@@ -84,11 +80,10 @@
 ;;  If `haskell-doc-chop-off-fctname' is non-nil the function name is not
 ;;  shown together with the type (default: nil).
 
-;;; Internals:
-;;  ==========
+;; Internals:
 
-;; `haskell-doc-mode' is implemented as a minor-mode. So, you can combine it
-;; with any other mode. To enable it just type
+;; `haskell-doc-mode' is implemented as a minor-mode.  So, you can combine it
+;; with any other mode.  To enable it just type
 ;;   M-x turn-on-haskell-doc-mode
 
 ;; These are the names of the functions that can be called directly by the
@@ -106,24 +101,22 @@
 ;;  `haskell-doc-check-active' ... check whether haskell-doc is active;
 ;;                                 Key: CTRL-c ESC-/
 
-;;; ToDo:
-;;  =====
+;; ToDo:
 
 ;;   - Fix byte-compile problems in `haskell-doc-prelude-types' for getArgs etc
-;;   - Write a parser for .hi files. Read library interfaces via this parser.
+;;   - Write a parser for .hi files.  Read library interfaces via this parser.
 ;;   - Indicate kind of object with colours
 ;;   - Handle multi-line types
 ;;   - Encode i-am-fct info in the alist of ids and types.
 
-;;; Bugs:
-;;  =====
+;; Bugs:
 
-;;   - Some prelude fcts aren't displayed properly. This might be due to a
+;;   - Some prelude fcts aren't displayed properly.  This might be due to a
 ;;     name clash of Haskell and Elisp functions (e.g. length) which
 ;;     confuses Emacs when reading `haskell-doc-prelude-types'
 
 ;;; Changelog:
-;;  ==========
+
 ;;  $Log: haskell-doc.el,v $
 ;;  Revision 1.30  2009/02/02 21:00:33  monnier
 ;;  (haskell-doc-imported-list): Don't add current buffer
@@ -313,7 +306,6 @@
 ;;
 
 ;;; Code:
-;;  =====
 
 ;;@menu
 ;;* Constants and Variables::
@@ -394,10 +386,10 @@ This variable is buffer-local.")
 (make-variable-buffer-local 'haskell-doc-mode)
 
 (defvar haskell-doc-mode-hook nil
- "Hook invoked when entering `haskell-doc-mode'.")
+  "Hook invoked when entering `haskell-doc-mode'.")
 
 (defvar haskell-doc-index nil
- "Variable holding an alist matching file names to fct-type alists.
+  "Variable holding an alist matching file names to fct-type alists.
 The function `haskell-doc-make-global-fct-index' rebuilds this variables
 \(similar to an `imenu' rescan\).
 This variable is buffer-local.")
@@ -439,9 +431,9 @@ This variable is buffer-local."
 (make-variable-buffer-local 'haskell-doc-show-user-defined)
 
 (defcustom haskell-doc-chop-off-context t
- "If non-nil eliminate the context part in a Haskell type."
+  "If non-nil eliminate the context part in a Haskell type."
   :group 'haskell-doc
- :type 'boolean)
+  :type 'boolean)
 
 (defcustom haskell-doc-chop-off-fctname nil
   "If non-nil omit the function name and show only the type."
@@ -454,7 +446,7 @@ This variable is buffer-local."
   :type 'boolean)
 
 (defvar haskell-doc-search-distance 40  ; distance in characters
- "*How far to search when looking for the type declaration of fct under cursor.")
+  "*How far to search when looking for the type declaration of fct under cursor.")
 
 ;;@node Variables, Prelude types, Mode Variable, Constants and Variables
 ;;@subsection Variables
@@ -541,41 +533,41 @@ It is probably best to manipulate this data structure with the commands
     ("as" . "import [qualified] modid [as modid] [impspec]")
     ("qualified" . "import [qualified] modid [as modid] [impspec]")
     ("hiding" . "hiding ( import1 , ... , importn [ , ] )"))
- "An alist of reserved identifiers.
+  "An alist of reserved identifiers.
 Each element is of the form (ID . DOC) where both ID and DOC are strings.
 DOC should be a concise single-line string describing the construct in which
 the keyword is used.")
 
 (eval-and-compile
-(defalias 'haskell-doc-split-string
-  (if (condition-case ()
-	  (split-string "" nil t)
-	(wrong-number-of-arguments nil))
-      'split-string
-    ;; copied from Emacs 22
-    (lambda (string &optional separators omit-nulls)
-      (let ((keep-nulls (not (if separators omit-nulls t)))
-	    (rexp (or separators "[ \f\t\n\r\v]+"))
-	    (start 0)
-	    notfirst
-	    (list nil))
-	(while (and (string-match rexp string
-				  (if (and notfirst
-					   (= start (match-beginning 0))
-					   (< start (length string)))
-				      (1+ start) start))
-		    (< start (length string)))
-	  (setq notfirst t)
-	  (if (or keep-nulls (< start (match-beginning 0)))
-	      (setq list
-		    (cons (substring string start (match-beginning 0))
-			  list)))
-	  (setq start (match-end 0)))
-	(if (or keep-nulls (< start (length string)))
-	    (setq list
-		  (cons (substring string start)
-			list)))
-	(nreverse list))))))
+  (defalias 'haskell-doc-split-string
+    (if (condition-case ()
+            (split-string "" nil t)
+          (wrong-number-of-arguments nil))
+        'split-string
+      ;; copied from Emacs 22
+      (lambda (string &optional separators omit-nulls)
+        (let ((keep-nulls (not (if separators omit-nulls t)))
+              (rexp (or separators "[ \f\t\n\r\v]+"))
+              (start 0)
+              notfirst
+              (list nil))
+          (while (and (string-match rexp string
+                                    (if (and notfirst
+                                             (= start (match-beginning 0))
+                                             (< start (length string)))
+                                        (1+ start) start))
+                      (< start (length string)))
+            (setq notfirst t)
+            (if (or keep-nulls (< start (match-beginning 0)))
+                (setq list
+                      (cons (substring string start (match-beginning 0))
+                            list)))
+            (setq start (match-end 0)))
+          (if (or keep-nulls (< start (length string)))
+              (setq list
+                    (cons (substring string start)
+                          list)))
+          (nreverse list))))))
 
 ;;@cindex haskell-doc-prelude-types
 
@@ -647,18 +639,18 @@ the keyword is used.")
              (val-decl-re
               (concat "^\\( +\\)?" idlist-re "[ \t\n]*::[ \t\n]*" type-re))
              (re (concat
-                 ;; 3 possibilities: a class decl, a data decl, or val decl.
-                 ;; First, let's match a class decl.
-                 "^class \\(?:.*=>\\)? *\\(.*[^ \t\n]\\)[ \t\n]*where"
+                  ;; 3 possibilities: a class decl, a data decl, or val decl.
+                  ;; First, let's match a class decl.
+                  "^class \\(?:.*=>\\)? *\\(.*[^ \t\n]\\)[ \t\n]*where"
 
-                 ;; Or a value decl:
-                 "\\|" val-decl-re
+                  ;; Or a value decl:
+                  "\\|" val-decl-re
 
-                 "\\|" ;; Or a data decl.  We only handle single-arm
-                 ;; datatypes with labels.
-                 "^data +\\([[:alnum:]][[:alnum:] ]*[[:alnum:]]\\)"
-                 " *=.*{\\([^}]+\\)}"
-                 ))
+                  "\\|" ;; Or a data decl.  We only handle single-arm
+                  ;; datatypes with labels.
+                  "^data +\\([[:alnum:]][[:alnum:] ]*[[:alnum:]]\\)"
+                  " *=.*{\\([^}]+\\)}"
+                  ))
              (re-class (concat "^[^ \t\n]\\|" re))
              curclass)
         (while (re-search-forward (if curclass re-class re) nil t)
@@ -683,7 +675,7 @@ the keyword is used.")
                                 (setq classes (substring classes 1 -1)))
                             (setq type (concat "(" curclass ", " classes
                                                ") => " type)))
-                      (setq type (concat curclass " => " type)))
+                        (setq type (concat curclass " => " type)))
                     ;; It's actually not an error: just a type annotation on
                     ;; some local variable.
                     ;; (error "Indentation outside a class in %s: %s"
@@ -1235,64 +1227,64 @@ URL is the URL of the online doc."
 ;;@cindex haskell-doc-strategy-ids
 
 (defvar haskell-doc-strategy-ids
- (list
-  '("par"  . "Done -> Done -> Done ; [infixr 0]")
-  '("seq"  . "Done -> Done -> Done ; [infixr 1]")
+  (list
+   '("par"  . "Done -> Done -> Done ; [infixr 0]")
+   '("seq"  . "Done -> Done -> Done ; [infixr 1]")
 
-  '("using"      . "a -> Strategy a -> a ; [infixl 0]")
-  '("demanding"  . "a -> Done -> a ; [infixl 0]")
-  '("sparking"   . "a -> Done -> a ; [infixl 0]")
+   '("using"      . "a -> Strategy a -> a ; [infixl 0]")
+   '("demanding"  . "a -> Done -> a ; [infixl 0]")
+   '("sparking"   . "a -> Done -> a ; [infixl 0]")
 
-  '(">||" . "Done -> Done -> Done ; [infixr 2]")
-  '(">|" .  "Done -> Done -> Done ; [infixr 3]")
-  '("$||" . "(a -> b) -> Strategy a -> a -> b ; [infixl 6]")
-  '("$|"  . "(a -> b) -> Strategy a -> a -> b ; [infixl 6]")
-  '(".|"  . "(b -> c) -> Strategy b -> (a -> b) -> (a -> c) ; [infixl 9]")
-  '(".||" . "(b -> c) -> Strategy b -> (a -> b) -> (a -> c) ; [infixl 9]")
-  '("-|"  . "(a -> b) -> Strategy b -> (b -> c) -> (a -> c) ; [infixl 9]")
-  '("-||" . "(a -> b) -> Strategy b -> (b -> c) -> (a -> c) ; [infixl 9]")
+   '(">||" . "Done -> Done -> Done ; [infixr 2]")
+   '(">|" .  "Done -> Done -> Done ; [infixr 3]")
+   '("$||" . "(a -> b) -> Strategy a -> a -> b ; [infixl 6]")
+   '("$|"  . "(a -> b) -> Strategy a -> a -> b ; [infixl 6]")
+   '(".|"  . "(b -> c) -> Strategy b -> (a -> b) -> (a -> c) ; [infixl 9]")
+   '(".||" . "(b -> c) -> Strategy b -> (a -> b) -> (a -> c) ; [infixl 9]")
+   '("-|"  . "(a -> b) -> Strategy b -> (b -> c) -> (a -> c) ; [infixl 9]")
+   '("-||" . "(a -> b) -> Strategy b -> (b -> c) -> (a -> c) ; [infixl 9]")
 
-  '("Done" . "type Done = ()")
-  '("Strategy" . "type Strategy a = a -> Done")
+   '("Done" . "type Done = ()")
+   '("Strategy" . "type Strategy a = a -> Done")
 
-  '("r0"    . "Strategy a")
-  '("rwhnf" . "Eval a => Strategy a")
-  '("rnf" . "Strategy a")
-  '("NFData" . "class Eval a => NFData a where rnf :: Strategy a")
-  '("NFDataIntegral" ."class (NFData a, Integral a) => NFDataIntegral a")
-  '("NFDataOrd" . "class (NFData a, Ord a) => NFDataOrd a")
+   '("r0"    . "Strategy a")
+   '("rwhnf" . "Eval a => Strategy a")
+   '("rnf" . "Strategy a")
+   '("NFData" . "class Eval a => NFData a where rnf :: Strategy a")
+   '("NFDataIntegral" ."class (NFData a, Integral a) => NFDataIntegral a")
+   '("NFDataOrd" . "class (NFData a, Ord a) => NFDataOrd a")
 
-  '("markStrat" . "Int -> Strategy a -> Strategy a")
+   '("markStrat" . "Int -> Strategy a -> Strategy a")
 
-  '("seqPair" . "Strategy a -> Strategy b -> Strategy (a,b)")
-  '("parPair" . "Strategy a -> Strategy b -> Strategy (a,b)")
-  '("seqTriple" . "Strategy a -> Strategy b -> Strategy c -> Strategy (a,b,c)")
-  '("parTriple" . "Strategy a -> Strategy b -> Strategy c -> Strategy (a,b,c)")
+   '("seqPair" . "Strategy a -> Strategy b -> Strategy (a,b)")
+   '("parPair" . "Strategy a -> Strategy b -> Strategy (a,b)")
+   '("seqTriple" . "Strategy a -> Strategy b -> Strategy c -> Strategy (a,b,c)")
+   '("parTriple" . "Strategy a -> Strategy b -> Strategy c -> Strategy (a,b,c)")
 
-  '("parList"  . "Strategy a -> Strategy [a]")
-  '("parListN"  . "(Integral b) => b -> Strategy a -> Strategy [a]")
-  '("parListNth"  . "Int -> Strategy a -> Strategy [a]")
-  '("parListChunk"  . "Int -> Strategy a -> Strategy [a]")
-  '("parMap"  . "Strategy b -> (a -> b) -> [a] -> [b]")
-  '("parFlatMap"  . "Strategy [b] -> (a -> [b]) -> [a] -> [b]")
-  '("parZipWith"  . "Strategy c -> (a -> b -> c) -> [a] -> [b] -> [c]")
-  '("seqList"  . "Strategy a -> Strategy [a]")
-  '("seqListN"  . "(Integral a) => a -> Strategy b -> Strategy [b]")
-  '("seqListNth"  . "Int -> Strategy b -> Strategy [b]")
+   '("parList"  . "Strategy a -> Strategy [a]")
+   '("parListN"  . "(Integral b) => b -> Strategy a -> Strategy [a]")
+   '("parListNth"  . "Int -> Strategy a -> Strategy [a]")
+   '("parListChunk"  . "Int -> Strategy a -> Strategy [a]")
+   '("parMap"  . "Strategy b -> (a -> b) -> [a] -> [b]")
+   '("parFlatMap"  . "Strategy [b] -> (a -> [b]) -> [a] -> [b]")
+   '("parZipWith"  . "Strategy c -> (a -> b -> c) -> [a] -> [b] -> [c]")
+   '("seqList"  . "Strategy a -> Strategy [a]")
+   '("seqListN"  . "(Integral a) => a -> Strategy b -> Strategy [b]")
+   '("seqListNth"  . "Int -> Strategy b -> Strategy [b]")
 
-  '("parBuffer"  . "Int -> Strategy a -> [a] -> [a]")
+   '("parBuffer"  . "Int -> Strategy a -> [a] -> [a]")
 
-  '("seqArr"  . "(Ix b) => Strategy a -> Strategy (Array b a)")
-  '("parArr"  . "(Ix b) => Strategy a -> Strategy (Array b a)")
+   '("seqArr"  . "(Ix b) => Strategy a -> Strategy (Array b a)")
+   '("parArr"  . "(Ix b) => Strategy a -> Strategy (Array b a)")
 
-  '("fstPairFstList"  . "(NFData a) => Strategy [(a,b)]")
-  '("force"  . "(NFData a) => a -> a ")
-  '("sforce"  . "(NFData a) => a -> b -> b")
-  )
- "Alist of strategy functions and their types as defined in Strategies.lhs.")
+   '("fstPairFstList"  . "(NFData a) => Strategy [(a,b)]")
+   '("force"  . "(NFData a) => a -> a ")
+   '("sforce"  . "(NFData a) => a -> b -> b")
+   )
+  "Alist of strategy functions and their types as defined in Strategies.lhs.")
 
 (defvar haskell-doc-user-defined-ids nil
- "Alist of functions and strings defined by the user.")
+  "Alist of functions and strings defined by the user.")
 
 ;;@node Test membership,  , Prelude types, Constants and Variables
 ;;@subsection Test membership
@@ -1348,18 +1340,18 @@ URL is the URL of the online doc."
   ;; Add the menu to the hugs menu as last entry.
   (let ((hugsmap (lookup-key (current-local-map) [menu-bar Hugs])))
     (if (not (or (featurep 'xemacs) ; XEmacs has problems here
-		 (not (keymapp hugsmap))
-		 (lookup-key hugsmap [haskell-doc])))
-	(if (functionp 'define-key-after)
-	    (define-key-after hugsmap [haskell-doc]
-	      (cons "Haskell-doc" haskell-doc-keymap)
-	      [Haskell-doc mode]))))
+                 (not (keymapp hugsmap))
+                 (lookup-key hugsmap [haskell-doc])))
+        (if (functionp 'define-key-after)
+            (define-key-after hugsmap [haskell-doc]
+              (cons "Haskell-doc" haskell-doc-keymap)
+              [Haskell-doc mode]))))
   ;; Add shortcuts for these commands.
   (local-set-key "\C-c\e/" 'haskell-doc-check-active)
   ;; Conflicts with the binding of haskell-insert-otherwise.
   ;; (local-set-key "\C-c\C-o" 'haskell-doc-mode)
   (local-set-key [(control shift meta mouse-3)]
-		 'haskell-doc-ask-mouse-for-type))
+                 'haskell-doc-ask-mouse-for-type))
 
 
 ;;@node Haskell Doc Mode, Switch it on or off, Menubar Support, top
@@ -1377,10 +1369,10 @@ See variable docstring."
   (interactive (list (or current-prefix-arg 'toggle)))
 
   (setq haskell-doc-mode
-	(cond
-	 ((eq arg 'toggle) (not haskell-doc-mode))
-	 (arg (> (prefix-numeric-value arg) 0))
-	 (t)))
+        (cond
+         ((eq arg 'toggle) (not haskell-doc-mode))
+         (arg (> (prefix-numeric-value arg) 0))
+         (t)))
 
   ;; First, unconditionally turn the mode OFF.
 
@@ -1410,7 +1402,7 @@ See variable docstring."
       (add-hook 'post-command-hook
                 'haskell-doc-mode-print-current-symbol-info nil 'local))
     (and haskell-doc-show-global-types
-	 (haskell-doc-make-global-fct-index)) ; build type index for global fcts
+         (haskell-doc-make-global-fct-index)) ; build type index for global fcts
 
     (haskell-doc-install-keymap)
 
@@ -1418,15 +1410,15 @@ See variable docstring."
 
   (and (called-interactively-p 'any)
        (message "haskell-doc-mode is %s"
-		(if haskell-doc-mode "enabled" "disabled")))
+                (if haskell-doc-mode "enabled" "disabled")))
   haskell-doc-mode)
 
 (defmacro haskell-doc-toggle-var (id prefix)
   ;; toggle variable or set it based on prefix value
   `(setq ,id
-	 (if ,prefix
-	     (>= (prefix-numeric-value ,prefix) 0)
-	   (not ,id))) )
+         (if ,prefix
+             (>= (prefix-numeric-value ,prefix) 0)
+           (not ,id))) )
 
 ;;@cindex haskell-doc-show-global-types
 (defun haskell-doc-show-global-types (&optional prefix)
@@ -1468,9 +1460,13 @@ See variable docstring."
 ;;;###autoload
 (defalias 'turn-on-haskell-doc-mode 'haskell-doc-mode)
 
-;;@cindex  turn-off-haskell-doc-mode
+;;;###autoload
+(defalias 'turn-on-haskell-doc 'haskell-doc-mode)
 
-(defun turn-off-haskell-doc-mode ()
+;;@cindex  turn-off-haskell-doc-mode
+(defalias 'turn-off-haskell-doc-mode 'turn-off-haskell-doc)
+
+(defun turn-off-haskell-doc ()
   "Unequivocally turn off `haskell-doc-mode' (which see)."
   (haskell-doc-mode 0))
 
@@ -1485,12 +1481,12 @@ Should be the same as the value of `haskell-doc-mode' but alas currently it
 is not."
   (interactive)
   (message "%s"
-   (if (or (and haskell-doc-mode haskell-doc-timer)
-           (memq 'haskell-doc-mode-print-current-symbol-info
-                 post-command-hook))
-       "haskell-doc is ACTIVE"
-     (substitute-command-keys
-      "haskell-doc is not ACTIVE \(Use \\[haskell-doc-mode] to turn it on\)"))))
+           (if (or (and haskell-doc-mode haskell-doc-timer)
+                   (memq 'haskell-doc-mode-print-current-symbol-info
+                         post-command-hook))
+               "haskell-doc is ACTIVE"
+             (substitute-command-keys
+              "haskell-doc is not ACTIVE \(Use \\[haskell-doc-mode] to turn it on\)"))))
 
 ;;@node Top level function, Mouse interface, Check, top
 ;;@section Top level function
@@ -1513,12 +1509,12 @@ This function is run by an idle timer to print the type
          (sit-for haskell-doc-idle-delay))
        ;; good morning! read the word under the cursor for breakfast
        (haskell-doc-show-type)))
-       ;; ;; ToDo: find surrounding fct
-       ;; (cond ((eq current-symbol current-fnsym)
-       ;;        (haskell-doc-show-type current-fnsym))
-       ;;       (t
-       ;;        (or nil ; (haskell-doc-print-var-docstring current-symbol)
-       ;;            (haskell-doc-show-type current-fnsym)))))))
+;; ;; ToDo: find surrounding fct
+;; (cond ((eq current-symbol current-fnsym)
+;;        (haskell-doc-show-type current-fnsym))
+;;       (t
+;;        (or nil ; (haskell-doc-print-var-docstring current-symbol)
+;;            (haskell-doc-show-type current-fnsym)))))))
 
 ;;;###autoload
 (defun haskell-doc-current-info ()
@@ -1532,14 +1528,14 @@ Meant for `eldoc-documentation-function'."
 
 ;;@cindex haskell-doc-ask-mouse-for-type
 (defun haskell-doc-ask-mouse-for-type (event)
- "Read the identifier under the mouse and echo its type.
+  "Read the identifier under the mouse and echo its type.
 This uses the same underlying function `haskell-doc-show-type' as the hooked
 function.  Only the user interface is different."
- (interactive "e")
- (save-excursion
-   (select-window (posn-window (event-end event)))
-   (goto-char (posn-point (event-end event)))
-   (haskell-doc-show-type)))
+  (interactive "e")
+  (save-excursion
+    (select-window (posn-window (event-end event)))
+    (goto-char (posn-point (event-end event)))
+    (haskell-doc-show-type)))
 
 
 ;;@node Print fctsym, Movement, Mouse interface, top
@@ -1669,7 +1665,7 @@ the haskell-doc database."
 ;;(defun haskell-doc-fnsym-in-current-sexp ()
 ;;  (let* ((p (point))
 ;;         (sym (progn
-;;		(forward-word -1)
+;;              (forward-word -1)
 ;;                (while (and (forward-word -1) ; (haskell-doc-forward-sexp-safe -1)
 ;;                            (> (point) (point-min))))
 ;;                (cond ((or (= (point) (point-min))
@@ -1685,7 +1681,7 @@ the haskell-doc database."
 ;;                           (error nil)))))))
 ;;    (goto-char p)
 ;;    (if sym
-;;	(format "%s" sym)
+;;      (format "%s" sym)
 ;;      sym)))
 
 ;;    (and (symbolp sym)
@@ -1698,72 +1694,72 @@ the haskell-doc database."
 
 ;;@cindex haskell-doc-grab-line
 (defun haskell-doc-grab-line (fct-and-pos)
- "Get the type of an \(FCT POSITION\) pair from the current buffer."
- ;; (if (null fct-and-pos)
- ;;     "" ; fn is not a local fct
+  "Get the type of an \(FCT POSITION\) pair from the current buffer."
+  ;; (if (null fct-and-pos)
+  ;;     "" ; fn is not a local fct
   (let ( (str ""))
-   (goto-char (cdr fct-and-pos))
-   (beginning-of-line)
-   ;; search for start of type (phsp give better bound?)
-   (if (null (search-forward "::" (+ (point) haskell-doc-search-distance) t))
-       ""
-     (setq str (haskell-doc-grab))        ; leaves point at end of line
-     (while (haskell-doc-wrapped-type-p)  ; while in a multi-line type expr
-       (forward-line 1)
-       (beginning-of-line)
-       (skip-chars-forward " \t")
-       (setq str (concat str (haskell-doc-grab))))
-     (haskell-doc-string-nub-ws           ; squeeze string
-      (if haskell-doc-chop-off-context    ; no context
-	  (haskell-doc-chop-off-context str)
-	str)))))
- ;; (concat (car fct-and-pos) "::" (haskell-doc-string-nub-ws str))))
+    (goto-char (cdr fct-and-pos))
+    (beginning-of-line)
+    ;; search for start of type (phsp give better bound?)
+    (if (null (search-forward "::" (+ (point) haskell-doc-search-distance) t))
+        ""
+      (setq str (haskell-doc-grab))        ; leaves point at end of line
+      (while (haskell-doc-wrapped-type-p)  ; while in a multi-line type expr
+        (forward-line 1)
+        (beginning-of-line)
+        (skip-chars-forward " \t")
+        (setq str (concat str (haskell-doc-grab))))
+      (haskell-doc-string-nub-ws           ; squeeze string
+       (if haskell-doc-chop-off-context    ; no context
+           (haskell-doc-chop-off-context str)
+         str)))))
+;; (concat (car fct-and-pos) "::" (haskell-doc-string-nub-ws str))))
 
 ;;@cindex haskell-doc-wrapped-type-p
 (defun haskell-doc-wrapped-type-p ()
- "Check whether the type under the cursor is wrapped over several lines.
+  "Check whether the type under the cursor is wrapped over several lines.
 The cursor must be at the end of a line, which contains the type.
 Currently, only the following is checked:
 If this line ends with a `->' or the next starts with an `->' it is a
 multi-line type \(same for `=>'\).
 `--' comments are ignored.
 ToDo: Check for matching parenthesis!."
- (save-excursion
-   (let ( (here (point))
-	  (lim (progn (beginning-of-line) (point)))
-	  ;; (foo "")
-	  (res nil)
-	  )
-   (goto-char here)
-   (search-backward "--" lim t) ; skip over `--' comment
-   (skip-chars-backward " \t")
-   (if (bolp)                   ; skip empty lines
-      (progn
-       (forward-line 1)
-       (end-of-line)
-       (setq res (haskell-doc-wrapped-type-p)))
-   (forward-char -1)
-   ;; (setq foo (concat foo (char-to-string (preceding-char)) (char-to-string (following-char))))
-   (if (or (and (or (char-equal (preceding-char) ?-) (char-equal (preceding-char) ?=))
-		(char-equal (following-char) ?>)) ; (or -!> =!>
-	   (char-equal (following-char) ?,))      ;     !,)
-       (setq res t)
-     (forward-line)
-     (let ((here (point)))
-       (goto-char here)
-       (skip-chars-forward " \t")
-       (if (looking-at "--")  ; it is a comment line
-	   (progn
-	     (forward-line 1)
-	     (end-of-line)
-	     (setq res (haskell-doc-wrapped-type-p)))
-	 (forward-char 1)
-	 ;; (setq foo (concat foo (char-to-string (preceding-char)) (char-to-string (following-char))))
-	 ;; (message "|%s|" foo)
-	 (if (and (or (char-equal (preceding-char) ?-) (char-equal (preceding-char) ?=))
-		  (char-equal (following-char) ?>)) ; -!> or =!>
-	     (setq res t))))))
-   res)))
+  (save-excursion
+    (let ( (here (point))
+           (lim (progn (beginning-of-line) (point)))
+           ;; (foo "")
+           (res nil)
+           )
+      (goto-char here)
+      (search-backward "--" lim t) ; skip over `--' comment
+      (skip-chars-backward " \t")
+      (if (bolp)                   ; skip empty lines
+          (progn
+            (forward-line 1)
+            (end-of-line)
+            (setq res (haskell-doc-wrapped-type-p)))
+        (forward-char -1)
+        ;; (setq foo (concat foo (char-to-string (preceding-char)) (char-to-string (following-char))))
+        (if (or (and (or (char-equal (preceding-char) ?-) (char-equal (preceding-char) ?=))
+                     (char-equal (following-char) ?>)) ; (or -!> =!>
+                (char-equal (following-char) ?,))      ;     !,)
+            (setq res t)
+          (forward-line)
+          (let ((here (point)))
+            (goto-char here)
+            (skip-chars-forward " \t")
+            (if (looking-at "--")  ; it is a comment line
+                (progn
+                  (forward-line 1)
+                  (end-of-line)
+                  (setq res (haskell-doc-wrapped-type-p)))
+              (forward-char 1)
+              ;; (setq foo (concat foo (char-to-string (preceding-char)) (char-to-string (following-char))))
+              ;; (message "|%s|" foo)
+              (if (and (or (char-equal (preceding-char) ?-) (char-equal (preceding-char) ?=))
+                       (char-equal (following-char) ?>)) ; -!> or =!>
+                  (setq res t))))))
+      res)))
 
 ;;@cindex haskell-doc-grab
 (defun haskell-doc-grab ()
@@ -1788,26 +1784,26 @@ ToDo: Also eliminate leading and trailing whitespace."
 ;;(defun haskell-doc-string-nub-ws (str)
 ;;  "Replace all sequences of whitespaces in STR by just one whitespace."
 ;;  (let ( (res "")
-;;	 (l (length str))
-;;	 (i 0)
-;;	 (j 0)
-;;	 (in-ws nil))
+;;       (l (length str))
+;;       (i 0)
+;;       (j 0)
+;;       (in-ws nil))
 ;;   (while (< i l)
 ;;     (let* ( (c (string-to-char (substring str i (1+ i))))
-;;	    (is-ws (eq (char-syntax c) ? )) )
+;;          (is-ws (eq (char-syntax c) ? )) )
 ;;       (if (not (and in-ws is-ws))
-;;	     (setq res (concat res (char-to-string c))))
+;;           (setq res (concat res (char-to-string c))))
 ;;       (setq in-ws is-ws)
 ;;       (setq i (1+ i))))
 ;;   res))
 
 ;;@cindex haskell-doc-chop-off-context
 (defun haskell-doc-chop-off-context (str)
- "Eliminate the context in a type represented by the string STR."
- (let ((i (string-match "=>" str)) )
-   (if (null i)
-       str
-     (substring str (+ i 2)))))
+  "Eliminate the context in a type represented by the string STR."
+  (let ((i (string-match "=>" str)) )
+    (if (null i)
+        str
+      (substring str (+ i 2)))))
 
 ;;@cindex haskell-doc-get-imenu-info
 (defun haskell-doc-get-imenu-info (obj kind)
@@ -1854,32 +1850,32 @@ ToDo: Also eliminate leading and trailing whitespace."
 ;;@cindex haskell-doc-rescan-files
 
 (defun haskell-doc-rescan-files (filelist)
- "Do an `imenu' rescan on every file in FILELIST and return the fct-list.
+  "Do an `imenu' rescan on every file in FILELIST and return the fct-list.
 This function switches to and potentially loads many buffers."
- (save-current-buffer
-   (mapcar (lambda (f)
-             (set-buffer (find-file-noselect f))
-             (imenu--make-index-alist)
-             (cons f
-                   (mapcar (lambda (x)
-                             `(,(car x) . ,(haskell-doc-grab-line x)))
-                           (cdr (assoc "Variables" imenu--index-alist)))))
-           filelist)))
+  (save-current-buffer
+    (mapcar (lambda (f)
+              (set-buffer (find-file-noselect f))
+              (imenu--make-index-alist)
+              (cons f
+                    (mapcar (lambda (x)
+                              `(,(car x) . ,(haskell-doc-grab-line x)))
+                            (cdr (assoc "Variables" imenu--index-alist)))))
+            filelist)))
 
 ;;@cindex haskell-doc-make-global-fct-index
 
 (defun haskell-doc-make-global-fct-index ()
- "Scan imported files for types of global fcts and update `haskell-doc-index'."
- (interactive)
- (setq haskell-doc-index
-       (haskell-doc-rescan-files (haskell-doc-imported-list))))
+  "Scan imported files for types of global fcts and update `haskell-doc-index'."
+  (interactive)
+  (setq haskell-doc-index
+        (haskell-doc-rescan-files (haskell-doc-imported-list))))
 
 ;; ToDo: use a separate munge-type function to format type concisely
 
 ;;@cindex haskell-doc-get-global-fct-type
 
 (defun haskell-doc-get-global-fct-type (&optional sym)
- "Get type for function symbol SYM by examining `haskell-doc-index'."
+  "Get type for function symbol SYM by examining `haskell-doc-index'."
   (interactive) ;  "fName of outer `include' file: \nsFct:")
   (save-excursion
     ;; (switch-to-buffer "*scratch*")
@@ -1887,24 +1883,24 @@ This function switches to and potentially loads many buffers."
     ;; ;; Produces a list of fct-type alists
     ;; (if (null sym)
     ;;     (setq sym (progn (forward-word -1) (read (current-buffer)))))
-  (or sym
-      (current-word))
-  (let* ( (fn sym) ; (format "%s" sym))
-	  (fal haskell-doc-index)
-	  (res "") )
-    (while (not (null fal))
-      (let* ( (l (car fal))
-	      (f (car l))
-	      (x (assoc fn (cdr l))) )
-	(if (not (null x))
-	    (let* ( (ty (cdr x)) ; the type as string
-		    (idx (string-match "::" ty))
-		    (str (if (null idx)
-			     ty
-			   (substring ty (+ idx 2)))) )
-	      (setq res (format "[%s] %s" f str))))
-	  (setq fal (cdr fal))))
-    res))) ; (message res)) )
+    (or sym
+        (current-word))
+    (let* ( (fn sym) ; (format "%s" sym))
+            (fal haskell-doc-index)
+            (res "") )
+      (while (not (null fal))
+        (let* ( (l (car fal))
+                (f (car l))
+                (x (assoc fn (cdr l))) )
+          (if (not (null x))
+              (let* ( (ty (cdr x)) ; the type as string
+                      (idx (string-match "::" ty))
+                      (str (if (null idx)
+                               ty
+                             (substring ty (+ idx 2)))) )
+                (setq res (format "[%s] %s" f str))))
+          (setq fal (cdr fal))))
+      res))) ; (message res)) )
 
 ;;@node Local fct type,  , Global fct type, Print fctsym
 ;;@subsection Local fct type
@@ -1912,27 +1908,27 @@ This function switches to and potentially loads many buffers."
 ;;@cindex haskell-doc-get-and-format-fct-type
 
 (defun haskell-doc-get-and-format-fct-type (fn)
- "Get the type and kind of FN by checking local and global functions."
- (save-excursion
-   (save-match-data
-     (let ((docstring "")
-	   (doc nil)
-	   )
-       ;; is it a local function?
-       (setq docstring (haskell-doc-get-imenu-info fn "Variables"))
-       (if (not (null docstring))
-		;; (string-match (format "^%s\\s-+::\\s-+\\(.*\\)$" fn) docstring))
-	   (setq doc `(,docstring . "Variables"))) ; `(,(match-string 1 docstring) . "Variables") ))
-       ;; is it a type declaration?
-       (setq docstring (haskell-doc-get-imenu-info fn "Types"))
-       (if (not (null docstring))
-		;; (string-match (format "^\\s-*type\\s-+%s.*$" fn) docstring))
-	     (setq doc `(,docstring . "Types"))) ; `(,(match-string 0 docstring) . "Types")) )
-       (if (not (null docstring))
-		;; (string-match (format "^\\s-*data.*%s.*$" fn) docstring))
-	 (setq doc `(,docstring . "Data"))) ; (setq doc `(,(match-string 0 docstring) . "Data")) )
-       ;; return the result
-       doc ))))
+  "Get the type and kind of FN by checking local and global functions."
+  (save-excursion
+    (save-match-data
+      (let ((docstring "")
+            (doc nil)
+            )
+        ;; is it a local function?
+        (setq docstring (haskell-doc-get-imenu-info fn "Variables"))
+        (if (not (null docstring))
+            ;; (string-match (format "^%s\\s-+::\\s-+\\(.*\\)$" fn) docstring))
+            (setq doc `(,docstring . "Variables"))) ; `(,(match-string 1 docstring) . "Variables") ))
+        ;; is it a type declaration?
+        (setq docstring (haskell-doc-get-imenu-info fn "Types"))
+        (if (not (null docstring))
+            ;; (string-match (format "^\\s-*type\\s-+%s.*$" fn) docstring))
+            (setq doc `(,docstring . "Types"))) ; `(,(match-string 0 docstring) . "Types")) )
+        (if (not (null docstring))
+            ;; (string-match (format "^\\s-*data.*%s.*$" fn) docstring))
+            (setq doc `(,docstring . "Data"))) ; (setq doc `(,(match-string 0 docstring) . "Data")) )
+        ;; return the result
+        doc ))))
 
 
 ;;@appendix
