@@ -9,7 +9,7 @@
 ;; Version: 1.2
 ;; URL: http://www.iro.umontreal.ca/~lapalme/layout/index.html
 
-;;; This file is not part of GNU Emacs.
+;; This file is not part of GNU Emacs.
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -148,7 +148,7 @@ Makes sure that the same indentation info is not pushed twice.
 Uses free var `haskell-indent-info'."
   (let ((tmp (cons col name)))
     (if (member tmp haskell-indent-info)
-	haskell-indent-info
+        haskell-indent-info
       (push tmp haskell-indent-info))))
 
 (defun haskell-indent-push-pos (pos &optional name)
@@ -163,7 +163,7 @@ Uses free var `haskell-indent-info'."
   (setq column (+ column offset))
   ;; (if (and haskell-indent-tab-align (> offset 0))
   ;;     (* 8 (/ (+ column 7) 8))
-    column) ;; )
+  column) ;; )
 
 (defun haskell-indent-push-pos-offset (pos &optional offset)
   "Pushes indentation information for the column corresponding to POS
@@ -354,8 +354,8 @@ take a coarser approximation and stop at the first empty line."
       ;; find the first line of code which is not a comment
       (forward-comment (point-max))
       (if (> (point) save-point)
-	  start-code
-	(point)))))
+          start-code
+        (point)))))
 
 (defun haskell-indent-open-structure (start end)
   "If any structure (list or tuple) is not closed, between START and END,
@@ -376,17 +376,17 @@ Returns the location of the start of the comment, nil otherwise."
   (let (pps)
     (assert (<= start end))
     (cond ((= start end) nil)
-	  ((nth 4 (save-excursion (setq pps (parse-partial-sexp start end))))
-	   (nth 8 pps))
-	  ;; We also want to say that we are *at* the beginning of a comment.
-	  ((and (not (nth 8 pps))
+          ((nth 4 (save-excursion (setq pps (parse-partial-sexp start end))))
+           (nth 8 pps))
+          ;; We also want to say that we are *at* the beginning of a comment.
+          ((and (not (nth 8 pps))
                 (>= (point-max) (+ end 2))
-		(nth 4 (save-excursion
-			 (setq pps (parse-partial-sexp end (+ end 2))))))
-	   (nth 8 pps)))))
+                (nth 4 (save-excursion
+                         (setq pps (parse-partial-sexp end (+ end 2))))))
+           (nth 8 pps)))))
 
 (defvar haskell-indent-off-side-keywords-re
-      "\\<\\(do\\|let\\|of\\|where\\|mdo\\|rec\\)\\>[ \t]*")
+  "\\<\\(do\\|let\\|of\\|where\\|mdo\\|rec\\)\\>[ \t]*")
 
 (defun haskell-indent-type-at-point ()
   "Return the type of the line (also puts information in `match-data')."
@@ -407,14 +407,14 @@ Returns the location of the start of the comment, nil otherwise."
   "Generate contour information between START and END points."
   (if (< start end)
       (save-excursion
-	(goto-char end)
-	(haskell-indent-skip-blanks-and-newlines-backward start)
-        (let ((cur-col (current-column))            ; maximum column number
-              (fl 0)     ; number of lines that forward-line could not advance
+        (goto-char end)
+        (haskell-indent-skip-blanks-and-newlines-backward start)
+        (let ((cur-col (current-column)) ; maximum column number
+              (fl 0) ; number of lines that forward-line could not advance
               contour)
           (while (and (> cur-col 0) (= fl 0) (>= (point) start))
             (haskell-indent-back-to-indentation)
-	    (if (< (point) start) (goto-char start))
+            (if (< (point) start) (goto-char start))
             (and (not (member (haskell-indent-type-at-point)
                               '(empty comment))) ; skip empty and comment lines
                  (< (current-column) cur-col) ; less indented column found
@@ -427,14 +427,14 @@ Returns the location of the start of the comment, nil otherwise."
   "Move point to the next symbol."
   (skip-syntax-forward ")" end)
   (if (< (point) end)
-     (progn
-       (forward-sexp 1)
-       (haskell-indent-skip-blanks-and-newlines-forward end))))
+      (progn
+        (forward-sexp 1)
+        (haskell-indent-skip-blanks-and-newlines-forward end))))
 
 (defun haskell-indent-next-symbol-safe (end)
   "Puts point to the next following symbol, or to end if there are no more symbols in the sexp."
   (condition-case errlist (haskell-indent-next-symbol end)
-      (error (goto-char end))))
+    (error (goto-char end))))
 
 (defun haskell-indent-separate-valdef (start end)
   "Return a list of positions for important parts of a valdef."
@@ -622,13 +622,13 @@ Returns the location of the start of the comment, nil otherwise."
              (haskell-indent-push-pos aft-rhs-sign))
           ;; "001.10"  10= gd rh
           (10 (if (haskell-indent-no-otherwise guard) (haskell-indent-push-pos guard "| "))
-	      (if last-line (haskell-indent-push-pos-offset guard)))
+              (if last-line (haskell-indent-push-pos-offset guard)))
           ;; "001100"  11= gd agd
           (11 (if (haskell-indent-no-otherwise guard) (haskell-indent-push-pos guard "| "))
-	      (haskell-indent-push-pos aft-guard))
+              (haskell-indent-push-pos aft-guard))
           ;; "001000"  12= gd
           (12 (if (haskell-indent-no-otherwise guard) (haskell-indent-push-pos guard "| "))
-	      (if last-line (haskell-indent-push-pos-offset guard 2)))
+              (if last-line (haskell-indent-push-pos-offset guard 2)))
           ;; "000011"  13= rh arh
           (13 (haskell-indent-push-pos aft-rhs-sign))
           ;; "000010"  14= rh
@@ -654,8 +654,10 @@ Returns the location of the start of the comment, nil otherwise."
          (is-where
           (string-match "where[ \t]*" haskell-indent-current-line-first-ident))
          (diff-first                 ; not a function def with the same name
-          (not(string= (haskell-trim valname-string)
-                       (haskell-trim haskell-indent-current-line-first-ident))))
+          (or (null valname-string)
+              (not (string= (haskell-trim valname-string)
+                            (haskell-trim haskell-indent-current-line-first-ident)))))
+
          ;; (is-type-def
          ;;  (and rhs-sign (eq (char-after rhs-sign) ?\:)))
          (test (string
@@ -775,49 +777,49 @@ than an identifier, a guard or rhs."
                (string-match haskell-indent-start-keywords-re valname-string))
           (haskell-indent-push-pos-offset valname)
         (case                           ; general case
-         (haskell-indent-find-case test)
-         ;; "1.1.11"   1= vn gd rh arh
-         (1 (haskell-indent-push-pos aft-rhs-sign))
-         ;; "1.1.10"   2= vn gd rh
-         (2 (if last-line
-                   (haskell-indent-push-pos-offset guard)
+            (haskell-indent-find-case test)
+          ;; "1.1.11"   1= vn gd rh arh
+          (1 (haskell-indent-push-pos aft-rhs-sign))
+          ;; "1.1.10"   2= vn gd rh
+          (2 (if last-line
+                 (haskell-indent-push-pos-offset guard)
                (haskell-indent-push-pos-offset rhs-sign 2)))
-         ;; "1.1100"   3= vn gd agd
-         (3 (haskell-indent-push-pos aft-guard))
-         ;; "1.1000"   4= vn gd
-         (4 (haskell-indent-push-pos-offset guard 2))
-         ;; "1.0011"   5= vn rh arh
-         (5 (haskell-indent-push-pos valname)
-            (haskell-indent-push-pos aft-rhs-sign))
-         ;; "1.0010"   6= vn rh
-         (6 (if last-line
-                (haskell-indent-push-pos-offset valname)
-              (haskell-indent-push-pos-offset rhs-sign 2)))
-         ;; "110000"   7= vn avn
-         (7 (haskell-indent-push-pos-offset aft-valname))
-         ;; "100000"   8= vn
-         (8 (haskell-indent-push-pos valname))
-         ;; "001.11"   9= gd rh arh
-         (9 (haskell-indent-push-pos aft-rhs-sign))
-         ;; "001.10"  10= gd rh
-         (10 (if last-line
-                   (haskell-indent-push-pos-offset guard)
+          ;; "1.1100"   3= vn gd agd
+          (3 (haskell-indent-push-pos aft-guard))
+          ;; "1.1000"   4= vn gd
+          (4 (haskell-indent-push-pos-offset guard 2))
+          ;; "1.0011"   5= vn rh arh
+          (5 (haskell-indent-push-pos valname)
+             (haskell-indent-push-pos aft-rhs-sign))
+          ;; "1.0010"   6= vn rh
+          (6 (if last-line
+                 (haskell-indent-push-pos-offset valname)
                (haskell-indent-push-pos-offset rhs-sign 2)))
-         ;; "001100"  11= gd agd
-         (11 (if (haskell-indent-no-otherwise guard)
-                   (haskell-indent-push-pos aft-guard)))
-         ;; "001000"  12= gd
-         (12 (if last-line (haskell-indent-push-pos-offset guard 2)))
-         ;; "000011"  13= rh arh
-         (13 (haskell-indent-push-pos aft-rhs-sign))
-         ;; "000010"  14= rh
-         (14 (if last-line (haskell-indent-push-pos-offset rhs-sign 2)))
-         ;; "000000"  15=
-         (t (error "haskell-indent-other: %s impossible case" test ))))
+          ;; "110000"   7= vn avn
+          (7 (haskell-indent-push-pos-offset aft-valname))
+          ;; "100000"   8= vn
+          (8 (haskell-indent-push-pos valname))
+          ;; "001.11"   9= gd rh arh
+          (9 (haskell-indent-push-pos aft-rhs-sign))
+          ;; "001.10"  10= gd rh
+          (10 (if last-line
+                  (haskell-indent-push-pos-offset guard)
+                (haskell-indent-push-pos-offset rhs-sign 2)))
+          ;; "001100"  11= gd agd
+          (11 (if (haskell-indent-no-otherwise guard)
+                  (haskell-indent-push-pos aft-guard)))
+          ;; "001000"  12= gd
+          (12 (if last-line (haskell-indent-push-pos-offset guard 2)))
+          ;; "000011"  13= rh arh
+          (13 (haskell-indent-push-pos aft-rhs-sign))
+          ;; "000010"  14= rh
+          (14 (if last-line (haskell-indent-push-pos-offset rhs-sign 2)))
+          ;; "000000"  15=
+          (t (error "haskell-indent-other: %s impossible case" test ))))
       haskell-indent-info)))
 
 (defun haskell-indent-valdef-indentation (start end end-visible curr-line-type
-                                          indent-info)
+                                                indent-info)
   "Find indentation information for a value definition."
   (let ((haskell-indent-info indent-info))
     (if (< start end-visible)
@@ -831,7 +833,7 @@ than an identifier, a guard or rhs."
       haskell-indent-info)))
 
 (defun haskell-indent-line-indentation (line-start line-end end-visible
-                                         curr-line-type indent-info)
+                                                   curr-line-type indent-info)
   "Compute indentation info between LINE-START and END-VISIBLE.
 Separate a line of program into valdefs between offside keywords
 and find indentation info for each part."
@@ -848,8 +850,8 @@ and find indentation info for each part."
                                      line-end t)
                   (not (or (haskell-indent-in-comment line-start (point))
                            (haskell-indent-in-string line-start (point)))))
-	(let ((beg-match (match-beginning 0)) ; save beginning of match
-	      (end-match (match-end 0)))      ; save end of match
+        (let ((beg-match (match-beginning 0)) ; save beginning of match
+              (end-match (match-end 0)))      ; save end of match
           ;; Do not try to find indentation points if off-side-keyword at
           ;; the start...
           (if (or (< line-start beg-match)
@@ -873,39 +875,39 @@ and find indentation info for each part."
 (defun haskell-indent-layout-indent-info (start contour-line)
   (let ((haskell-indent-info nil)
         (curr-line-type (haskell-indent-type-at-point))
-	line-start line-end end-visible)
+        line-start line-end end-visible)
     (save-excursion
       (if (eq curr-line-type 'ident)
-	  (let				; guess the type of line
-	      ((sep
-		(haskell-indent-separate-valdef
-		 (point) (line-end-position))))
-	    ;; if the first ident is where or the start of a def
-	    ;; keep it in a global variable
-	    (setq haskell-indent-current-line-first-ident
-		  (if (string-match "where[ \t]*" (nth 1 sep))
-		      (nth 1 sep)
-		    (if (nth 5 sep)		; is there a rhs-sign
-			(if (= (char-after (nth 5 sep)) ?\:) ;is it a typdef
-			    "::" (nth 1 sep))
-		      "")))))
-      (while contour-line		; explore the contour points
-	(setq line-start (pop contour-line))
-	(goto-char line-start)
-	(setq line-end (line-end-position))
-	(setq end-visible		; visible until the column of the
-	      (if contour-line		; next contour point
-		  (save-excursion
-		    (move-to-column
-		     (haskell-indent-point-to-col (car contour-line)))
-		    (point))
-		line-end))
-	(unless (or (haskell-indent-open-structure start line-start)
-		    (haskell-indent-in-comment start line-start))
-	  (setq haskell-indent-info
-		(haskell-indent-line-indentation line-start line-end
-						 end-visible curr-line-type
-						 haskell-indent-info)))))
+          (let                          ; guess the type of line
+              ((sep
+                (haskell-indent-separate-valdef
+                 (point) (line-end-position))))
+            ;; if the first ident is where or the start of a def
+            ;; keep it in a global variable
+            (setq haskell-indent-current-line-first-ident
+                  (if (string-match "where[ \t]*" (nth 1 sep))
+                      (nth 1 sep)
+                    (if (nth 5 sep)              ; is there a rhs-sign
+                        (if (= (char-after (nth 5 sep)) ?\:) ;is it a typdef
+                            "::" (nth 1 sep))
+                      "")))))
+      (while contour-line               ; explore the contour points
+        (setq line-start (pop contour-line))
+        (goto-char line-start)
+        (setq line-end (line-end-position))
+        (setq end-visible            ; visible until the column of the
+              (if contour-line       ; next contour point
+                  (save-excursion
+                    (move-to-column
+                     (haskell-indent-point-to-col (car contour-line)))
+                    (point))
+                line-end))
+        (unless (or (haskell-indent-open-structure start line-start)
+                    (haskell-indent-in-comment start line-start))
+          (setq haskell-indent-info
+                (haskell-indent-line-indentation line-start line-end
+                                                 end-visible curr-line-type
+                                                 haskell-indent-info)))))
     haskell-indent-info))
 
 (defun haskell-indent-find-matching-start (regexp limit &optional pred start)
@@ -959,8 +961,8 @@ OPEN is the start position of the comment in which point is."
   ;; whether it's text.  Instead, we'll assume it's text.
   (save-excursion
     (if (= open (point))
-	;; We're actually just in front of a comment: align with following
-	;; code or with comment on previous line.
+        ;; We're actually just in front of a comment: align with following
+        ;; code or with comment on previous line.
         (let ((prev-line-info
                (cond
                 ((eq (char-after) ?\{) nil) ;Align as if it were code.
@@ -997,15 +999,15 @@ OPEN is the start position of the comment in which point is."
 
       ;; We really are inside a comment.
       (if (looking-at "-}")
-	  (progn
-	    (forward-char 2)
-	    (forward-comment -1)
+          (progn
+            (forward-char 2)
+            (forward-comment -1)
             (list (list (1+ (haskell-indent-point-to-col (point))))))
-	(let ((offset (if (looking-at "--?")
-			  (- (match-beginning 0) (match-end 0)))))
-	  (forward-line -1)		;Go to previous line.
-	  (haskell-indent-back-to-indentation)
-	  (if (< (point) start) (goto-char start))
+        (let ((offset (if (looking-at "--?")
+                          (- (match-beginning 0) (match-end 0)))))
+          (forward-line -1)             ;Go to previous line.
+          (haskell-indent-back-to-indentation)
+          (if (< (point) start) (goto-char start))
 
           (list (list (if (looking-at comment-start-skip)
                           (if offset
@@ -1068,10 +1070,10 @@ is at the end of an otherwise-non-empty line."
   :group 'haskell-indent
   :type '(repeat (choice string
                          (cons :tag "" (string :tag "keyword:")
-                         (cons :tag "" (integer :tag "offset")
-                         (choice (const nil)
-                                 (list :tag ""
-                                       (integer :tag "offset-pending"))))))))
+                               (cons :tag "" (integer :tag "offset")
+                                     (choice (const nil)
+                                             (list :tag ""
+                                                   (integer :tag "offset-pending"))))))))
 
 (defun haskell-indent-skip-lexeme-forward ()
   (and (zerop (skip-syntax-forward "w"))
@@ -1143,18 +1145,18 @@ is at the end of an otherwise-non-empty line."
     ;; There might still be layout within the open structure.
     (let* ((end (point))
            (basic-indent-info
-             ;; Anything else than a ) is subject to layout.
-             (if (looking-at "\\s.\\|\\$ ")
-                 (haskell-indent-point-to-col open) ; align a punct with (
-               (let ((follow (save-excursion
-                               (goto-char (1+ open))
-                               (haskell-indent-skip-blanks-and-newlines-forward end)
-                               (point))))
-                 (if (= follow end)
-                     (save-excursion
-                       (goto-char open)
-                       (haskell-indent-after-keyword-column nil nil 1))
-                   (haskell-indent-point-to-col follow)))))
+            ;; Anything else than a ) is subject to layout.
+            (if (looking-at "\\s.\\|\\$ ")
+                (haskell-indent-point-to-col open) ; align a punct with (
+              (let ((follow (save-excursion
+                              (goto-char (1+ open))
+                              (haskell-indent-skip-blanks-and-newlines-forward end)
+                              (point))))
+                (if (= follow end)
+                    (save-excursion
+                      (goto-char open)
+                      (haskell-indent-after-keyword-column nil nil 1))
+                  (haskell-indent-point-to-col follow)))))
            (open-column (haskell-indent-point-to-col open))
            (contour-line (haskell-indent-contour-line (1+ open) end)))
       (if (null contour-line)
@@ -1256,38 +1258,38 @@ We stay in the cycle as long as the TAB key is pressed."
       ;; use the ordinary tab for text...
       (funcall (default-value 'indent-line-function))
     (let ((marker (if (> (current-column) (haskell-indent-current-indentation))
-		      (point-marker)))
-	  (bol (progn (beginning-of-line) (point))))
+                      (point-marker)))
+          (bol (progn (beginning-of-line) (point))))
       (haskell-indent-back-to-indentation)
       (unless (and (eq last-command this-command)
-		   (eq bol (car haskell-indent-last-info)))
-	(save-excursion
-	  (setq haskell-indent-last-info
-		(list bol (haskell-indent-indentation-info) 0 0))))
+                   (eq bol (car haskell-indent-last-info)))
+        (save-excursion
+          (setq haskell-indent-last-info
+                (list bol (haskell-indent-indentation-info) 0 0))))
 
       (let* ((il (nth 1 haskell-indent-last-info))
-	     (index (nth 2 haskell-indent-last-info))
-	     (last-insert-length (nth 3 haskell-indent-last-info))
-	     (indent-info (nth index il)))
+             (index (nth 2 haskell-indent-last-info))
+             (last-insert-length (nth 3 haskell-indent-last-info))
+             (indent-info (nth index il)))
 
-	(haskell-indent-line-to (car indent-info)) ; insert indentation
-	(delete-char last-insert-length)
-	(setq last-insert-length 0)
-	(let ((text (cdr indent-info)))
-	  (if text
-	      (progn
-		(insert text)
-		(setq last-insert-length (length text)))))
+        (haskell-indent-line-to (car indent-info)) ; insert indentation
+        (delete-char last-insert-length)
+        (setq last-insert-length 0)
+        (let ((text (cdr indent-info)))
+          (if text
+              (progn
+                (insert text)
+                (setq last-insert-length (length text)))))
 
-	(setq haskell-indent-last-info
-	      (list bol il (% (1+ index) (length il)) last-insert-length))
+        (setq haskell-indent-last-info
+              (list bol il (% (1+ index) (length il)) last-insert-length))
 
-	(if (= (length il) 1)
-	    (message "Sole indentation")
-	  (message "Indent cycle (%d)..." (length il)))
+        (if (= (length il) 1)
+            (message "Sole indentation")
+          (message "Indent cycle (%d)..." (length il)))
 
-	(if marker
-	    (goto-char (marker-position marker)))))))
+        (if marker
+            (goto-char (marker-position marker)))))))
 
 (defun haskell-indent-region (start end)
   (error "Auto-reindentation of a region is not supported"))
@@ -1321,23 +1323,23 @@ If P-ARG is t align all defs up to the mark.
 TYPE is either 'guard or 'rhs."
   (save-excursion
     (let (start-block end-block
-          (maxcol (if (eq type 'rhs) haskell-indent-rhs-align-column 0))
-          contour sep defname defnamepos
-          defcol pos lastpos
-          regstack eqns-start start-found)
+                      (maxcol (if (eq type 'rhs) haskell-indent-rhs-align-column 0))
+                      contour sep defname defnamepos
+                      defcol pos lastpos
+                      regstack eqns-start start-found)
       ;; find the starting and ending boundary points for alignment
       (if p-arg
           (if (mark)                    ; aligning everything in the region
-            (progn
-              (when (> (mark) (point)) (exchange-point-and-mark))
-              (setq start-block
-                    (save-excursion
-                      (goto-char (mark))
-                      (line-beginning-position)))
-              (setq end-block
-                  (progn (if (haskell-indent-bolp)
-                             (haskell-indent-forward-line -1))
-                         (line-end-position))))
+              (progn
+                (when (> (mark) (point)) (exchange-point-and-mark))
+                (setq start-block
+                      (save-excursion
+                        (goto-char (mark))
+                        (line-beginning-position)))
+                (setq end-block
+                      (progn (if (haskell-indent-bolp)
+                                 (haskell-indent-forward-line -1))
+                             (line-end-position))))
             (error "The mark is not set for aligning definitions"))
         ;; aligning the current definition
         (setq start-block (haskell-indent-start-of-def))
@@ -1391,13 +1393,13 @@ TYPE is either 'guard or 'rhs."
             ;; go through each equation to find the region to indent
             (while eqns-start
               (let ((eqn (caar eqns-start)))
-		(setq lastpos (if (cdr eqns-start)
-				  (save-excursion
-				    (goto-char (caadr eqns-start))
-				    (haskell-indent-forward-line -1)
-				    (line-end-position))
-				end-block))
-		(setq sep (haskell-indent-separate-valdef eqn lastpos)))
+                (setq lastpos (if (cdr eqns-start)
+                                  (save-excursion
+                                    (goto-char (caadr eqns-start))
+                                    (haskell-indent-forward-line -1)
+                                    (line-end-position))
+                                end-block))
+                (setq sep (haskell-indent-separate-valdef eqn lastpos)))
               (if (eq type 'guard)
                   (setq pos (nth 3 sep))
                 ;; check if what follows a rhs sign is more indented or not
@@ -1451,7 +1453,7 @@ TYPE is either 'guard or 'rhs."
 Alignment works only if all guards are to the south-east of their |."
   (interactive "*")
   (let ((pc (if (haskell-indent-bolp) ?\012
-                (preceding-char)))
+              (preceding-char)))
         (pc1 (or (char-after (- (point) 2)) 0)))
     ;; check what guard to insert depending on the previous context
     (if (= pc ?\ )                      ; x = any char other than blank or |
