@@ -1,4 +1,4 @@
-;;; inf-haskell.el --- Interaction with an inferior Haskell process.
+;;; inf-haskell.el --- Interaction with an inferior Haskell process
 
 ;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
 
@@ -31,7 +31,7 @@
 ;;; Code:
 
 (require 'comint)
-(require 'shell)			;For directory tracking.
+(require 'shell)             ; For directory tracking.
 (require 'compile)
 (require 'haskell-mode)
 (require 'haskell-decl-scan)
@@ -98,8 +98,8 @@ The command can include arguments."
           ;; Foo.hs:318:80:
           ;;     Ambiguous occurrence `Bar'
           ;;     It could refer to either `Bar', defined at Zork.hs:311:5
-          ;; 		          or `Bar', imported from Bars at Frob.hs:32:0-16
-          ;; 				       (defined at Location.hs:97:5)
+          ;;                  or `Bar', imported from Bars at Frob.hs:32:0-16
+          ;;                       (defined at Location.hs:97:5)
           ("[ (]defined at \\(.+\\):\\([0-9]+\\):\\([0-9]+\\))?$" 1 2 3 0)
           ("imported from .* at \\(.+\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\)$"
            1 2 (3 . 4) 0)
@@ -119,7 +119,7 @@ This will either look for a Cabal file or a \"module\" statement in the file."
   :group 'inferior-haskell
   (set (make-local-variable 'comint-prompt-regexp)
        ;; Whay the backslash in [\\._[:alnum:]]?
-       "^\\*?[[:upper:]][\\._[:alnum:]]*\\(?: \\*?[[:upper:]][\\._[:alnum:]]*\\)*> \\|^> $")
+       "^\\*?[[:upper:]][\\._[:alnum:]]*\\(?: \\*?[[:upper:]][\\._[:alnum:]]*\\)*> \\|^λ?> $")
   (set (make-local-variable 'comint-input-autoexpand) nil)
   (add-hook 'comint-preoutput-filter-functions
             'inferior-haskell-send-decl-post-filter)
@@ -157,12 +157,12 @@ This will either look for a Cabal file or a \"module\" statement in the file."
 (defun inferior-haskell-string-to-strings (string)
   "Split the STRING into a list of strings."
   (let ((i (string-match "[\"]" string)))
-    (if (null i) (split-string string)	; no quoting:  easy
+    (if (null i) (split-string string) ; no quoting:  easy
       (append (unless (eq i 0) (split-string (substring string 0 i)))
-	      (let ((rfs (read-from-string string i)))
-		(cons (car rfs)
-		      (inferior-haskell-string-to-strings
-		       (substring string (cdr rfs)))))))))
+              (let ((rfs (read-from-string string i)))
+                (cons (car rfs)
+                      (inferior-haskell-string-to-strings
+                       (substring string (cdr rfs)))))))))
 
 (defun inferior-haskell-command (arg)
   (inferior-haskell-string-to-strings
@@ -180,19 +180,19 @@ It runs the hook `inferior-haskell-hook' after starting the process and
 setting up the inferior-haskell buffer."
   (interactive (list (inferior-haskell-command current-prefix-arg)))
   (setq inferior-haskell-buffer
-	(apply 'make-comint "haskell" (car command) nil (cdr command)))
+        (apply 'make-comint "haskell" (car command) nil (cdr command)))
   (with-current-buffer inferior-haskell-buffer
     (inferior-haskell-mode)
     (run-hooks 'inferior-haskell-hook)))
 
 (defun inferior-haskell-process (&optional arg)
   (or (if (buffer-live-p inferior-haskell-buffer)
-	  (get-buffer-process inferior-haskell-buffer))
+          (get-buffer-process inferior-haskell-buffer))
       (progn
-	(let ((current-prefix-arg arg))
-	  (call-interactively 'inferior-haskell-start-process))
-	;; Try again.
-	(inferior-haskell-process arg))))
+        (let ((current-prefix-arg arg))
+          (call-interactively 'inferior-haskell-start-process))
+        ;; Try again.
+        (inferior-haskell-process arg))))
 
 ;;;###autoload
 (defalias 'run-haskell 'switch-to-haskell)
@@ -216,11 +216,11 @@ setting up the inferior-haskell buffer."
              #1=(string-match inferior-haskell-multiline-prompt-re string))
     ;; deleting sequence of `%s|' multiline promts
     (while #1#
-      (setq string (substring string (match-end 0))))    
+      (setq string (substring string (match-end 0))))
     ;; deleting regular prompts
     (setq string (replace-regexp-in-string comint-prompt-regexp "" string)
           ;; turning off this post-filter
-          inferior-haskell-send-decl-post-filter-on nil))  
+          inferior-haskell-send-decl-post-filter-on nil))
   string)
 
 (defvar inferior-haskell-seen-prompt nil)
@@ -323,7 +323,7 @@ If prefix arg \\[universal-argument] is given, just reload the previous file."
   (save-buffer)
   (let ((buf (current-buffer))
         (file buffer-file-name)
-	(proc (inferior-haskell-process)))
+        (proc (inferior-haskell-process)))
     (if file
         (with-current-buffer (process-buffer proc)
           (compilation-forget-errors)
